==> kite-1.0b6.tar.gz: kite-1.0b6/modules/__internal/_regex.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "regex/microregex.h"

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_construct) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *regex = kite_string_object(thd, arg1);

    this->builtin_data.filevalue = 
        (FILE*)microregex_create(KITE_GET_STRING_VALUE(regex));
    kite_dereference_object(regex);
}

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_destruct) {
    KITE_NO_ARGS;
    (void)thd;
    microregex_destroy((microregex_t)this->builtin_data.filevalue);
}

/*****************************************************************************
 * Set case insensitivity
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_set_insensitive)
{
    kite_object_t *arg1 = kite_boolean_object(
        thd, args->builtin_data.listvalue.car
    );

    microregex_set_case_insensitivity(
        (microregex_t)this->builtin_data.filevalue,
        KITE_GET_INTEGER(arg1)
    );

    kite_dereference_object(arg1);
}

/*****************************************************************************
 * Match on regex.
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_match) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *arg2 = 
        args->builtin_data.listvalue.cdr ?
        args->builtin_data.listvalue.cdr->builtin_data.listvalue.car :
        NULL;
    kite_object_t *string = kite_string_object(thd, arg1);
    kite_object_t *start = 
        arg2 != NULL ?
        kite_int_object(thd, arg2) :
        kite_new_integer(thd, 0);

    microregex_state_t retmatch = 
        microregex_match(
            (microregex_t)this->builtin_data.filevalue,
            KITE_GET_STRING_VALUE(string),
            KITE_GET_INTEGER(start)
        );

    kite_object_t *ret;
    if (retmatch) 
    {
        kite_object_t *cls = 
            kite_dereference_and_load(thd, "__internal._regex_match");
        ret = kite_new_instance(thd, cls);
        ret->builtin_data.filevalue = retmatch;
    }
    else
    {
        ret = kite_new_null(thd);
    }
    
    kite_dereference_object(string);
    kite_dereference_object(start);
    kite_vm_push(thd, ret);
}

KITE_MODULE_INITIALIZER(_regex) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_regex");
    kite_set_docstring(newclass, "Regular expression internal code.");
    
    /* Add methods. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_with_docs(thread, FUNC_COMPILED, Regex_construct, 
                                  "Class constructor.", 1,
                                  "rgx", "Regular expression to create."));
    kite_add_method(thread, newclass, "match", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Regex_match, 
                                  "Matches regular expression to text.", 1,
                                  "text", "Text to match on."));
    kite_add_method(thread, newclass, "match", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Regex_match, 
                                  "Matches regular expression to text.", 2,
                                  "text", "Text to match on.",
                                  "start", "Position in string to start from (0-based)."));
    kite_add_method(thread, newclass, "set_insensitive",
        kite_new_method_with_docs(thread, FUNC_COMPILED, Regex_set_insensitive, 
                                  "Sets case insensitivity of regular expression.", 1,
                                  "val", "Value to set to (true/false)."));

    newclass->object_data.destructor =
        kite_new_method_with_docs(thread, FUNC_COMPILED, Regex_destruct,
                                  "Destroys regular expression object.", 0);
        
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_regex", 1, NULL);
    kite_set_property(thread, parent, "_regex", newclass);
}

==> kite-1.0.0.tar.gz: kite-1.0.0/modules/__internal/_regex.c <==
/*****************************************************************************
 * Copyright (c) 2008, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "kite_object.h"
#include "vm/kite_vm.h"
#include "regex/microregex.h"

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_construct) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *regex = kite_string_object(thd, arg1);

    this->builtin_data.filevalue = 
        (FILE*)microregex_create(KITE_GET_STRING_VALUE(regex));
    kite_dereference_object(regex);
}

/*****************************************************************************
 * Class destructor
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_destruct) {
    KITE_NO_ARGS;
    (void)thd;
    microregex_destroy((microregex_t*)&this->builtin_data.filevalue);
}

/*****************************************************************************
 * Set case insensitivity
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_set_insensitive)
{
    kite_object_t *arg1 = kite_boolean_object(
        thd, args->builtin_data.listvalue.car
    );

    microregex_set_case_insensitivity(
        (microregex_t)this->builtin_data.filevalue,
        KITE_GET_INTEGER(arg1)
    );

    kite_dereference_object(arg1);
}

/*****************************************************************************
 * Match on regex.
 ****************************************************************************/
KITE_CLASS_METHOD(Regex_match) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *arg2 = 
        args->builtin_data.listvalue.cdr ?
        args->builtin_data.listvalue.cdr->builtin_data.listvalue.car :
        NULL;
    kite_object_t *string = kite_string_object(thd, arg1);
    kite_object_t *start = 
        arg2 != NULL ?
        kite_int_object(thd, arg2) :
        kite_new_integer(thd, 0);

    microregex_state_t retmatch = 
        microregex_match(
            (microregex_t)this->builtin_data.filevalue,
            KITE_GET_STRING_VALUE(string),
            KITE_GET_INTEGER(start)
        );

    kite_object_t *ret;
    if (retmatch) 
    {
        kite_object_t *cls = 
            kite_dereference_and_load(thd, "__internal._regex_match");
        ret = kite_new_instance(thd, cls);
        ret->builtin_data.filevalue = retmatch;
    }
    else
    {
        ret = kite_new_null(thd);
    }
    
    kite_dereference_object(string);
    kite_dereference_object(start);
    kite_vm_push(thd, ret);
}

KITE_MODULE_INITIALIZER(_regex) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits = 
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = 
        kite_new_class(thread, inherits, "_regex");
    kite_set_docstring(newclass, "Regular expression internal code.");
    
    /* Add methods. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_compiled_with_docs(thread, Regex_construct, 
                                  "Class constructor.", 1,
                                  "rgx", "Regular expression to create."));
    kite_add_method(thread, newclass, "match", 
        kite_new_method_compiled_with_docs(thread, Regex_match, 
                                  "Matches regular expression to text.", 1,
                                  "text", "Text to match on."));
    kite_add_method(thread, newclass, "match", 
        kite_new_method_compiled_with_docs(thread, Regex_match, 
                                  "Matches regular expression to text.", 2,
                                  "text", "Text to match on.",
                                  "start", "Position in string to start from (0-based)."));
    kite_add_method(thread, newclass, "set_insensitive",
        kite_new_method_compiled_with_docs(thread, Regex_set_insensitive, 
                                  "Sets case insensitivity of regular expression.", 1,
                                  "val", "Value to set to (true/false)."));

    kite_add_method(thread, newclass, "__destruct__",
        kite_new_method_compiled_with_docs(thread, Regex_destruct,
                                  "Destroys regular expression object.", 0));
        
    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "_regex", 1, NULL);
    kite_set_property(thread, parent, "_regex", newclass);
}
